/*
   This file is part of libTAU,
   copyright: SAMBA group, Tel-Aviv Univ. Israel, 1997.

   Distribution in binary form only is permitted for use with IMP and
   MultiFit, by permission from Prof. Haim Wolfson.
*/

#ifndef TAU_SYMM_PROG_PARAMS_H
#define TAU_SYMM_PROG_PARAMS_H
#include "GridParams.h"
#include <vector>
#include <string>
#include "Vector3.h"
#include "tau_config.h"
namespace TAU {
struct TAUEXPORT LogParams {
  LogParams():intermediate_fn_(""),model_fn_("asmb.model"){}
  std::string intermediate_fn_;
  std::string output_fn_;
  std::string model_fn_;
};
struct TAUEXPORT SymmBaseParams {
  SymmBaseParams(): min_base_dist(4.0), max_base_dist(13.0) {}
  bool add(const std::string str);
  float min_base_dist;
  float max_base_dist;
};
struct TAUEXPORT ClusterParams {
  ClusterParams() : maxAngleDiff(0.1), ignoreClustersSmaller(2), rmsd(2.0) {}
  bool add(const std::string str);
  float maxAngleDiff;
  int ignoreClustersSmaller;
  float rmsd;
};
struct TAUEXPORT ScoreParams {
  ScoreParams() : small_interface_ratio(0.4), max_penetration(-2.5), ns_thr(0.6), weights(5,0) {}
  bool add(const std::string str);
  float small_interface_ratio;
  float max_penetration;
  float ns_thr;
  std::vector<int> weights;
};
struct TAUEXPORT SymmetryParams {
  SymmetryParams() : symmetryOf(3), steps(60){}
  bool add(const std::string str);
  int symmetryOf;
  int steps;
};
struct TAUEXPORT ActiveSiteParams {
  ActiveSiteParams() : active_site_filename(), matching_use(0), scoring_ratio(0.0) {}
  bool add(const std::string str);
  std::string active_site_filename;
  int matching_use;
  float scoring_ratio;
  void show(std::ostream &s=std::cout) {
    s<<"active_site_filename_:"<<active_site_filename<<std::endl;
    s<<"matching_use_:"<<matching_use<<std::endl;
    s<<"scoring_ratio_:"<<scoring_ratio<<std::endl;
  }
};
struct TAUEXPORT DockingParams {
  DockingParams() {}
  SymmBaseParams baseParams;
  ClusterParams clusterParams;
  ScoreParams scoreParams;
  SymmetryParams symmetryParams;
  ActiveSiteParams activeSiteParams;
  void show(std::ostream &s=std::cout) {
    activeSiteParams.show(s);
  }
};
struct TAUEXPORT DensityParams {
  DensityParams():non_symm_axis_length_(0.) {}
  std::string map_fn_;
  float res_;
  float thr_;
  float spacing_;
  Vector3 origin_;
  float pca_matching_thr_;
  float non_symm_axis_length_;
  void show(std::ostream &s=std::cout) const{
    s<<"map_fn:"<<map_fn_<<std::endl;
    s<<"resolution:"<<res_<<std::endl;
    s<<"threshold:"<<thr_<<std::endl;
    s<<"spacing:"<<spacing_<<std::endl;
    s<<"origin:";
    origin_.show(s);
    s<<std::endl;
    s<<"pca_matching_thr:"<<pca_matching_thr_<<std::endl;
    s<<"non_symm_axis_length:"<<non_symm_axis_length_<<std::endl;
  }
};
struct TAUEXPORT FittingParams {
  FittingParams():num_sols_(10) {}
  bool add(const std::string &str);
  void show(std::ostream &s=std::cout) const{
    s<<"num_sols:"<<num_sols_<<std::endl;
  }
  int num_sols_;
};
struct TAUEXPORT AlgParams {
  AlgParams() : protLib("chem.lib") {}
  GridParams gridParams;
  DockingParams dockingParams;
  DensityParams densityParams;
  FittingParams fittingParams;
  LogParams logParams;
  std::string unitPdb;
  std::string unitMs;
  std::string refPdb;
  std::string protLib;
  void show(std::ostream &s=std::cout) {
    s<<"unit_pdb:"<<unitPdb<<std::endl;
    s<<"unit_ms:"<<unitMs<<std::endl;
    s<<"ref_pdb:"<<refPdb<<std::endl;
    s<<"num fits:"<<fittingParams.num_sols_<<std::endl;
  }
};
class TAUEXPORT SymmProgParams {
 public:
  SymmProgParams(const char* paramFile);
  bool processParameters();
  const AlgParams& getAlgParams() const { return params; }
  inline std::string get_unit_pdb_fn() const {return unitPdb;}
  inline std::string get_unit_surface_fn() const {return unitMs;}
  inline std::string get_output_filename() const {
    return params.logParams.output_fn_;}
  inline std::string get_solution_model_filename() const {
    return params.logParams.model_fn_;}
  inline int get_number_of_solution_to_fit() const {
    return params.fittingParams.num_sols_;}
  inline std::string get_density_map_filename() const {
    return params.densityParams.map_fn_;}
  inline Vector3 get_density_origin() const {
    return params.densityParams.origin_;}
  inline float get_density_map_resolution() const {return params.densityParams.res_;}
  inline float get_density_map_threshold() const {return params.densityParams.thr_;}
  inline float get_pca_matching_threshold() const {return params.densityParams.pca_matching_thr_;}
  inline float get_density_map_spacing() const {return params.densityParams.spacing_;}
  inline float get_density_non_symm_axis_length() const {
    return params.densityParams.non_symm_axis_length_;}
  void set_density_non_symm_axis_length(float v) {
    params.densityParams.non_symm_axis_length_=v;}
  inline int get_cn_symm() const {return params.dockingParams.symmetryParams.symmetryOf;}
  inline int get_dn_symm() const {return dn_symm_deg_;}
  void show(std::ostream& out=std::cout) {
    params.show(out);
  }
private:
  void getParameters();
  bool parseParametersStrings();
private:
  AlgParams params;
  int dn_symm_deg_;
  std::string unitPdb;
  std::string unitMs;
  std::string refPdb;
  std::string protLib;
  std::string grid;
  std::string density_data_;
  std::string fitting_data_;    
  std::string baseParams;
  std::string matchingParams;
  std::string scoreParams;
  std::string symmetryParams;
  std::string clusterParams;
  std::string activeSiteParams;
};
}
#endif
